//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormSettings.h"
#include "FormAccount.h"
#include <portaudio.h>
#include <mmsystem.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmSettings *frmSettings;

namespace
{
	bool devListInitialized = false;
	std::vector<AnsiString> portaudioDevsIn;
	std::vector<AnsiString> portaudioDevsOut;
	std::vector<AnsiString> winwaveDevsIn;
	std::vector<AnsiString> winwaveDevsOut;
	void RefreshDevList(void)
	{
		portaudioDevsIn.clear();
		portaudioDevsOut.clear();

		PaError error;
		error = Pa_Initialize();
		if (paNoError == error)
		{

			int n = Pa_GetDeviceCount();

			for (int i=0; i<n; i++)
			{
				const PaDeviceInfo *info = Pa_GetDeviceInfo(i);
				if (info->maxInputChannels > 0)
				{
					portaudioDevsIn.push_back(info->name);
				}
				if (info->maxOutputChannels > 0)
				{
                	portaudioDevsOut.push_back(info->name);
				}
			}

			Pa_Terminate();	// paired with Pa_Initialize()		
		}
		else
		{
			//DEBUG_WARNING("PortAudio init: %s\n", Pa_GetErrorText(error));
		}

		winwaveDevsIn.clear();
		winwaveDevsOut.clear();

		WAVEOUTCAPS woc;
		int nOutDevices = waveOutGetNumDevs();
		//if (nOutDevices > 0)
		//	winwaveDevsOut.push_back("WAVE mapper");
		for (int i=0; i<nOutDevices; i++)
		{
			if (waveOutGetDevCaps(i, &woc, sizeof(WAVEOUTCAPS))==MMSYSERR_NOERROR)
			{
				winwaveDevsOut.push_back(woc.szPname);
			}
		}

		WAVEINCAPS wic;
		int nInDevices = waveInGetNumDevs();
		//if (nInDevices > 0)
		//	winwaveDevsIn.push_back("WAVE mapper");
		for (int i=0; i<nInDevices; i++)
		{
			if (waveInGetDevCaps(i, &wic, sizeof(WAVEINCAPS))==MMSYSERR_NOERROR)
			{
				winwaveDevsIn.push_back(wic.szPname);
			}
		}

	}
}

inline void strncpyz(char* dst, const char* src, int dstsize)
{
	strncpy(dst, src, dstsize);
	dst[dstsize-1] = '\0';
}

//---------------------------------------------------------------------------
__fastcall TfrmSettings::TfrmSettings(TComponent* Owner)
	: TForm(Owner)
{
	pcGeneral->ActivePage = tsGeneral;
}
//---------------------------------------------------------------------------

void TfrmSettings::FillDevList(TComboBox *target, int module, bool out, AnsiString selected)
{
	target->Items->Clear();
	std::vector<AnsiString> *v = NULL;
	if (module == 0)
	{
		if (out)
			v = &portaudioDevsOut;
		else
			v = &portaudioDevsIn;
	}
	else if (module == 1)
	{
		if (out)
			v = &winwaveDevsOut;
		else
			v = &winwaveDevsIn;
	}
	assert(v);
	for (int i=0; i<v->size(); i++)
	{
		AnsiString dev = v->at(i);
		target->Items->Add(dev);
		if (dev == selected)
		{
        	target->ItemIndex = i;
		}
	}
}

void __fastcall TfrmSettings::FormShow(TObject *Sender)
{
	if (devListInitialized == false)
	{
		RefreshDevList();
	}

	tmpSettings = appSettings;

	if (!strcmp(tmpSettings.uaConf.audioCfgSrc.mod, UaConf::modPortaudio))
	{
		cbSoundInputMod->ItemIndex = 0;
	}
	else
	{
		cbSoundInputMod->ItemIndex = 1;
	}
	FillDevList(cbSoundInputDev, cbSoundInputMod->ItemIndex, false, tmpSettings.uaConf.audioCfgSrc.dev);
	if (!strcmp(tmpSettings.uaConf.audioCfgPlay.mod, UaConf::modPortaudio))
	{
		cbSoundOutputMod->ItemIndex = 0;
	}
	else
	{
		cbSoundOutputMod->ItemIndex = 1;
	}
	FillDevList(cbSoundOutputDev, cbSoundOutputMod->ItemIndex, true, tmpSettings.uaConf.audioCfgPlay.dev);
	if (!strcmp(tmpSettings.uaConf.audioCfgAlert.mod, UaConf::modPortaudio))
	{
		cbSoundAlertOutputMod->ItemIndex = 0;
	}
	else
	{
		cbSoundAlertOutputMod->ItemIndex = 1;
	}
	FillDevList(cbSoundAlertOutputDev, cbSoundAlertOutputMod->ItemIndex, true, tmpSettings.uaConf.audioCfgAlert.dev);
	if (!strcmp(tmpSettings.uaConf.audioCfgPlayIntercom.mod, UaConf::modPortaudio))
	{
		cbSoundOutputIntercomMod->ItemIndex = 0;
	}
	else
	{
		cbSoundOutputIntercomMod->ItemIndex = 1;
	}
	FillDevList(cbSoundOutputIntercomDev, cbSoundOutputIntercomMod->ItemIndex, true, tmpSettings.uaConf.audioCfgPlayIntercom.dev);

	cbAec->ItemIndex = tmpSettings.uaConf.aec;
	edLocalAddress->Text = tmpSettings.uaConf.local.c_str();

	for (int i=0; i<tmpSettings.uaConf.accounts.size(); i++)
	{
		frmAccount = new TfrmAccount(NULL, i, tmpSettings.uaConf.accounts[i]);
		frmAccount->Parent = frmSettings->scrbAccounts;
		frmAccount->Visible = true;
		// force realign
		frmAccount->Align = alBottom;
		frmAccount->Align = alTop;
		frmAccountVector.push_back(frmAccount);
	}

    edGuiScaling->Text = tmpSettings.gui.scalingPct;

	chbAlwaysOnTop->Checked = tmpSettings.frmMain.bAlwaysOnTop;
	chbStartMinimizedToTray->Checked = tmpSettings.frmMain.bStartMinimizedToTray;
	chbTrayNotifierShowOnIncoming->Checked = tmpSettings.frmTrayNotifier.showOnIncoming;
	chbSkipTrayNotifierIfMainWindowVisible->Checked = tmpSettings.frmTrayNotifier.skipIfMainWindowVisible;

	cbSpeedDialSize->ItemIndex = tmpSettings.frmMain.iSpeedDialSize;

	chbLogToFile->Checked = tmpSettings.Logging.bLogToFile;
	chbLogMessages->Checked = tmpSettings.uaConf.logMessages;
	cmbMaxUiLogLines->ItemIndex = -1;
	for (int i=0; i<cmbMaxUiLogLines->Items->Count; i++)
	{
		if ((unsigned int)StrToInt(cmbMaxUiLogLines->Items->Strings[i]) >= tmpSettings.Logging.iMaxUiLogLines)
		{
			cmbMaxUiLogLines->ItemIndex = i;
			break;
		}
	}
	if (cmbMaxUiLogLines->ItemIndex == -1)
	{
		cmbMaxUiLogLines->ItemHeight = cmbMaxUiLogLines->Items->Count - 1;
	}

	chbG711->Checked = tmpSettings.uaConf.codecs.g711;
	chbG726_32->Checked = tmpSettings.uaConf.codecs.g726_32;
	chbGsm->Checked = tmpSettings.uaConf.codecs.gsm;
	chbSpeex->Checked = tmpSettings.uaConf.codecs.speex;

	chbAutoAnswer->Checked = tmpSettings.uaConf.autoAnswer;
	edAutoAnswerCode->Text = tmpSettings.uaConf.autoAnswerCode;
	edAutoAnswerDelayMin->Text = tmpSettings.uaConf.autoAnswerDelayMin;
	edAutoAnswerDelayMax->Text = tmpSettings.uaConf.autoAnswerDelayMax;

	chbAutoAnswerCallInfo->Checked = tmpSettings.uaConf.autoAnswerCallInfo;
	edAutoAnswerCallInfoDelayMin->Text = tmpSettings.uaConf.autoAnswerCallInfoDelayMin;

	edWebRtcAecMsInSndCardBuf->Text = tmpSettings.uaConf.webrtcAec.msInSndCardBuf;
	edWebRtcAecSkew->Text = tmpSettings.uaConf.webrtcAec.skew;
}
//---------------------------------------------------------------------------
void __fastcall TfrmSettings::btnCancelClick(TObject *Sender)
{
	this->Close();	
}
//---------------------------------------------------------------------------
void __fastcall TfrmSettings::btnApplyClick(TObject *Sender)
{
	tmpSettings.gui.scalingPct = StrToIntDef(edGuiScaling->Text, 100);
	if (tmpSettings.gui.scalingPct < tmpSettings.gui.SCALING_MIN ||
		tmpSettings.gui.scalingPct > tmpSettings.gui.SCALING_MAX)
	{
		tmpSettings.gui.scalingPct = 100;
	}

	tmpSettings.frmMain.bAlwaysOnTop = chbAlwaysOnTop->Checked;
	tmpSettings.frmMain.bStartMinimizedToTray = chbStartMinimizedToTray->Checked;
	tmpSettings.frmTrayNotifier.showOnIncoming = chbTrayNotifierShowOnIncoming->Checked;
	tmpSettings.frmTrayNotifier.skipIfMainWindowVisible = chbSkipTrayNotifierIfMainWindowVisible->Checked;

	tmpSettings.frmMain.iSpeedDialSize = cbSpeedDialSize->ItemIndex;

	tmpSettings.Logging.bLogToFile = chbLogToFile->Checked;
	tmpSettings.Logging.iMaxUiLogLines = StrToInt(cmbMaxUiLogLines->Text);	
	tmpSettings.uaConf.logMessages = chbLogMessages->Checked;

	const char* ptr = NULL;
	if (cbSoundInputMod->ItemIndex == 0)
	{
		ptr = UaConf::modPortaudio;
	}
	else
	{
		ptr = UaConf::modWinwave;
	}
	strncpyz(tmpSettings.uaConf.audioCfgSrc.mod, ptr, sizeof(tmpSettings.uaConf.audioCfgSrc.mod));
	strncpyz(tmpSettings.uaConf.audioCfgSrc.dev, cbSoundInputDev->Text.c_str(), sizeof(tmpSettings.uaConf.audioCfgSrc.dev));

	if (cbSoundOutputMod->ItemIndex == 0)
	{
		ptr = UaConf::modPortaudio;
	}
	else
	{
		ptr = UaConf::modWinwave;
	}
	strncpyz(tmpSettings.uaConf.audioCfgPlay.mod, ptr, sizeof(tmpSettings.uaConf.audioCfgPlay.mod));
	strncpyz(tmpSettings.uaConf.audioCfgPlay.dev, cbSoundOutputDev->Text.c_str(), sizeof(tmpSettings.uaConf.audioCfgPlay.dev));

	if (cbSoundAlertOutputMod->ItemIndex == 0)
	{
		ptr = UaConf::modPortaudio;
	}
	else
	{
		ptr = UaConf::modWinwave;
	}
	strncpyz(tmpSettings.uaConf.audioCfgAlert.mod, ptr, sizeof(tmpSettings.uaConf.audioCfgAlert.mod));
	strncpyz(tmpSettings.uaConf.audioCfgAlert.dev, cbSoundAlertOutputDev->Text.c_str(), sizeof(tmpSettings.uaConf.audioCfgAlert.dev));

	if (cbSoundOutputIntercomMod->ItemIndex == 0)
	{
		ptr = UaConf::modPortaudio;
	}
	else
	{
		ptr = UaConf::modWinwave;
	}
	strncpyz(tmpSettings.uaConf.audioCfgPlayIntercom.mod, ptr, sizeof(tmpSettings.uaConf.audioCfgPlayIntercom.mod));
	strncpyz(tmpSettings.uaConf.audioCfgPlayIntercom.dev, cbSoundOutputIntercomDev->Text.c_str(), sizeof(tmpSettings.uaConf.audioCfgPlayIntercom.dev));


	tmpSettings.uaConf.aec = (UaConf::Aec)cbAec->ItemIndex;

	tmpSettings.uaConf.local = edLocalAddress->Text.c_str();

	tmpSettings.uaConf.codecs.g711 = chbG711->Checked;
	tmpSettings.uaConf.codecs.g726_32 = chbG726_32->Checked;
	tmpSettings.uaConf.codecs.gsm = chbGsm->Checked;
	tmpSettings.uaConf.codecs.speex = chbSpeex->Checked;

   	tmpSettings.uaConf.autoAnswer = chbAutoAnswer->Checked;
	tmpSettings.uaConf.autoAnswerCode = StrToIntDef(edAutoAnswerCode->Text, 200);
	tmpSettings.uaConf.autoAnswerDelayMin = StrToIntDef(edAutoAnswerDelayMin->Text, 0);
	tmpSettings.uaConf.autoAnswerDelayMax = StrToIntDef(edAutoAnswerDelayMax->Text, 0);
	if (tmpSettings.uaConf.autoAnswerDelayMin > tmpSettings.uaConf.autoAnswerDelayMax)
	{
		tmpSettings.uaConf.autoAnswerDelayMin = 0;
		tmpSettings.uaConf.autoAnswerDelayMax = 0;		
	}

	tmpSettings.uaConf.autoAnswerCallInfo = chbAutoAnswerCallInfo->Checked;
	int val = StrToIntDef(edAutoAnswerCallInfoDelayMin->Text, 0);
	if (val < 0)
		val = 0;
	tmpSettings.uaConf.autoAnswerCallInfoDelayMin = val;


	tmpSettings.uaConf.webrtcAec.msInSndCardBuf = StrToIntDef(edWebRtcAecMsInSndCardBuf->Text, 120);
	if (tmpSettings.uaConf.webrtcAec.msInSndCardBuf < 0) {
    	tmpSettings.uaConf.webrtcAec.msInSndCardBuf = 120;
	}
	tmpSettings.uaConf.webrtcAec.skew = StrToIntDef(edWebRtcAecSkew->Text, 0);

	appSettings = tmpSettings;
	this->Close();	
}
//---------------------------------------------------------------------------



void __fastcall TfrmSettings::btnAddAccountClick(TObject *Sender)
{
	UaConf::Account acc;
	tmpSettings.uaConf.accounts.push_back(acc);
	int i = tmpSettings.uaConf.accounts.size() - 1;

	//scrbAccounts->AutoScroll = false;

	frmAccount = new TfrmAccount(NULL, i, tmpSettings.uaConf.accounts[i]);
	frmAccount->Parent = scrbAccounts;
	frmAccount->Visible = true;
	// force realign
	frmAccount->Align = alBottom;
	frmAccount->Align = alTop;

	//scrbAccounts->AutoScroll = true;

	// scroll to newly added account position (to the end)
	//scrbAccounts->ScrollBy(0, scrbAccounts->Height);
	frmAccount->SetFocus();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::FormClose(TObject *Sender, TCloseAction &Action)
{
	for (int i=0; i<frmAccountVector.size(); i++)
	{
		delete frmAccountVector[i];
	}
	frmAccountVector.clear();
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::cbSoundInputModChange(TObject *Sender)
{
	FillDevList(cbSoundInputDev, cbSoundInputMod->ItemIndex, false,
		tmpSettings.uaConf.audioCfgSrc.dev);
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::cbSoundOutputModChange(TObject *Sender)
{
	FillDevList(cbSoundOutputDev, cbSoundOutputMod->ItemIndex, true,
		tmpSettings.uaConf.audioCfgPlay.dev);
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::cbSoundAlertOutputModChange(TObject *Sender)
{
	FillDevList(cbSoundAlertOutputDev, cbSoundAlertOutputMod->ItemIndex, true,
		tmpSettings.uaConf.audioCfgAlert.dev);
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::cbSoundOutputIntercomModChange(TObject *Sender)
{
	FillDevList(cbSoundOutputIntercomDev, cbSoundOutputIntercomMod->ItemIndex, true,
		tmpSettings.uaConf.audioCfgPlayIntercom.dev);
}
//---------------------------------------------------------------------------

